@@ -1,10 +1,10 @@
 /* -*- mode: c; style: linux -*- */
 
 /* capplet-dir.c
- * Copyright (C) 2000 Helix Code, Inc.
+ * Copyright (C) 2000, 2001 Ximian, Inc.
  * Copyright (C) 1998 Red Hat Software, Inc.
  *
- * Written by Bradford Hovinen <hovinen@helixcode.com>,
+ * Written by Bradford Hovinen <hovinen@ximian.com>,
  *            Jonathan Blandford <jrb@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -45,7 +45,9 @@ static void capplet_dir_shutdown (CappletDir *capplet_dir);
 
 static GSList *read_entries (CappletDir *dir);
 
-static void start_capplet_through_root_manager (GnomeDesktopEntry *gde);
+#ifdef USE_ROOT_MANAGER
+static void start_capplet_through_root_manager (GnomeDesktopItem *gde);
+#endif
 
 CappletDirView *(*get_view_cb) (CappletDir *dir, CappletDirView *launcher);
 
@@ -57,8 +59,9 @@ capplet_new (CappletDir *dir, gchar *desktop_path)
 {
 	Capplet *capplet;
 	CappletDirEntry *entry;
-	GnomeDesktopEntry *dentry;
-	gchar *path;
+	GnomeDesktopItem *dentry;
+	gchar *path, **vec;
+	const gchar *execstr;
 
 	g_return_val_if_fail (desktop_path != NULL, NULL);
 
@@ -67,13 +70,20 @@ capplet_new (CappletDir *dir, gchar *desktop_path)
 		return entry;
 	}
 
-	dentry = gnome_desktop_entry_load (desktop_path);
+	dentry = gnome_desktop_item_new_from_file (desktop_path,
+						   GNOME_DESKTOP_ITEM_TYPE_NULL,
+						   NULL);
 	if (dentry == NULL)
 		return NULL;
 
-	if ((dentry->exec_length == 0) || !(path = gnome_is_program_in_path (dentry->exec[0])))
+	execstr = gnome_desktop_item_get_string (dentry,
+			GNOME_DESKTOP_ITEM_EXEC);
+	/* Perhaps use poptParseArgvString here */
+	vec = g_strsplit (execstr, " ", 0);
+	if (!(execstr && execstr[0]) || !(vec && (path = gnome_is_program_in_path (vec[0]))))
 	{
-		gnome_desktop_entry_free (dentry);
+		g_strfreev (vec);
+		gnome_desktop_item_unref (dentry);
 		return NULL;
 	}
 	g_free (path);
@@ -86,17 +96,48 @@ capplet_new (CappletDir *dir, gchar *desktop_path)
 	entry->type = TYPE_CAPPLET;
 	entry->entry = dentry;
 
-	entry->label = entry->entry->name;
-	entry->icon = entry->entry->icon;
-	entry->path = entry->entry->location;
+	entry->label = g_strdup (gnome_desktop_item_get_string (dentry,
+			GNOME_DESKTOP_ITEM_NAME));
+	entry->icon = g_strdup (gnome_desktop_item_get_string (dentry, GNOME_DESKTOP_ITEM_ICON));
 
+	if (entry->icon[0] == 0) {
+		g_free(entry->icon);
+		entry->icon = NULL;
+	}
+	
+	if (entry->icon) {
+		if (entry->icon[0] != '/')
+		{
+			gchar *old = entry->icon;
+			entry->icon = g_concat_dir_and_file (PIXMAPS_DIR, old);
+			g_free (old);
+		}
+		if (!g_file_exists (entry->icon) || g_file_test(entry->icon, G_FILE_TEST_IS_DIR))
+		{
+			const gchar *icon;
+			g_free (entry->icon);
+			icon = gnome_desktop_item_get_string (dentry, GNOME_DESKTOP_ITEM_ICON);
+			if (icon)
+				entry->icon = gnome_pixmap_file (icon);
+	
+			if (!entry->icon)
+				entry->icon = gnome_pixmap_file ("gnome-unknown.png");
+		}
+	} else {
+		entry->icon = gnome_program_locate_file
+			(gnome_program_get (), GNOME_FILE_DOMAIN_APP_PIXMAP,
+			 "control-center.png", TRUE, NULL);
+	}
+
+	if (!entry->icon) { /* if entry->icon still NULL */
+		entry->icon = gnome_pixmap_file ("gnome-unknown.png");
+	}
+
+	entry->pb = gdk_pixbuf_new_from_file (entry->icon, NULL);
+	entry->path = g_strdup (gnome_desktop_item_get_location (dentry));
+	entry->exec = vec;
 	entry->dir = dir;
 
-	if (!entry->icon)
-		entry->icon = GNOMECC_PIXMAPS_DIR "/control-center.png";
-
-	entry->pb = gdk_pixbuf_new_from_file (entry->icon);
-
 	g_hash_table_insert (capplet_hash, g_strdup (desktop_path), entry);
 
 	return entry;
@@ -122,20 +163,28 @@ capplet_dir_new (CappletDir *dir, gchar *dir_path)
 	entry = CAPPLET_DIR_ENTRY (capplet_dir);
 
 	entry->type = TYPE_CAPPLET_DIR;
-	entry->entry = gnome_desktop_entry_load (desktop_path);
+	entry->entry = gnome_desktop_item_new_from_file (desktop_path,
+			GNOME_DESKTOP_ITEM_TYPE_NULL,
+			NULL);
 	entry->dir = dir;
 	entry->path = g_strdup (dir_path);
 
 	g_free (desktop_path);
 
 	if (entry->entry) {
-		entry->label = entry->entry->name;
-		entry->icon = entry->entry->icon;
+		entry->label = g_strdup (gnome_desktop_item_get_string (
+				entry->entry,
+				GNOME_DESKTOP_ITEM_NAME));
+		entry->icon = g_strdup (gnome_desktop_item_get_string (
+				entry->entry,
+				GNOME_DESKTOP_ITEM_ICON));
 
 		if (!entry->icon)
-			entry->icon = GNOMECC_PIXMAPS_DIR "/control-center.png";
+			entry->icon = gnome_program_locate_file
+				(gnome_program_get (), GNOME_FILE_DOMAIN_APP_PIXMAP,
+				 "control-center.png", TRUE, NULL);
 
-		entry->pb = gdk_pixbuf_new_from_file (entry->icon);
+		entry->pb = gdk_pixbuf_new_from_file (entry->icon, NULL);
 	} else {
 		/* If the .directory file could not be found or read, abort */
 		g_free (capplet_dir);
@@ -164,10 +213,13 @@ capplet_dir_entry_destroy (CappletDirEntry *entry)
 		capplet_shutdown (CAPPLET (entry));
 	} else {
 		capplet_dir_shutdown (CAPPLET_DIR (entry));
-		g_free (entry->path);
 	}
 
-	gnome_desktop_entry_free (entry->entry);
+	g_free (entry->label);
+	g_free (entry->icon);
+	g_free (entry->path);
+	g_strfreev (entry->exec);
+	gnome_desktop_item_unref (entry->entry);
 	g_free (entry);
 }
 
@@ -206,32 +258,16 @@ capplet_reset_cb (Capplet *capplet)
 static void
 capplet_activate (Capplet *capplet) 
 {
-	GnomeDesktopEntry *entry;
+	CappletDirEntry *entry;
 
-	entry = CAPPLET_DIR_ENTRY (capplet)->entry;
+	entry = CAPPLET_DIR_ENTRY (capplet);
 
-#warning FIXME: this should probably be root-manager-helper
-#ifdef HAVE_BONOBO
-	if (!strncmp (entry->exec[0], "gnomecc", strlen ("gnomecc"))) {
-		if (capplet->launching) {
-			return;
-		} else {
-			capplet->launching = TRUE;
-			gtk_idle_add ((GtkFunction) capplet_reset_cb, capplet);
-			capplet_control_launch (entry->exec[2], entry->name);
-		}
-	} else
-#endif
-	if (!strncmp (entry->exec[0], "root-manager", strlen ("root-manager"))) {
-		start_capplet_through_root_manager (entry);
+	if (capplet->launching) {
+		return;
 	} else {
-		if (capplet->launching) {
-			return;
-		} else {
-			capplet->launching = TRUE;
-			gtk_timeout_add (1000, (GtkFunction) capplet_reset_cb, capplet);
-			gnome_desktop_entry_launch (entry);
-		}
+		capplet->launching = TRUE;
+		gtk_timeout_add (1000, (GtkFunction) capplet_reset_cb, capplet);
+		gnome_desktop_item_launch (entry->entry, NULL, 0, NULL);
 	}
 }
 
@@ -278,8 +314,8 @@ capplet_dir_shutdown (CappletDir *capplet_dir)
 static gint 
 node_compare (gconstpointer a, gconstpointer b) 
 {
-	return strcmp (CAPPLET_DIR_ENTRY (a)->entry->name, 
-		       CAPPLET_DIR_ENTRY (b)->entry->name);
+	return strcmp (CAPPLET_DIR_ENTRY (a)->label, 
+		       CAPPLET_DIR_ENTRY (b)->label);
 }
 
 /* Adapted from the original control center... */
@@ -338,8 +374,9 @@ read_entries (CappletDir *dir)
 		: list;
 }
 
+#ifdef USE_ROOT_MANAGER
 static void
-start_capplet_through_root_manager (GnomeDesktopEntry *gde) 
+start_capplet_through_root_manager (GnomeDesktopItem *gde) 
 {
 	static FILE *output = NULL;
 	pid_t pid;
@@ -385,6 +422,7 @@ start_capplet_through_root_manager (GnomeDesktopEntry *gde)
 	fflush (output);
 	g_free (cmdline);
 }
+#endif
 
 void 
 capplet_dir_init (CappletDirView *(*cb) (CappletDir *, CappletDirView *)) 
@@ -411,131 +449,3 @@ get_root_capplet_dir (void)
 
 	return root_dir;
 }
-
-static void
-capplet_ok_cb (GtkWidget *widget, GtkWidget *app) 
-{
-	CORBA_Environment ev;
-	Bonobo_PropertyControl pc;
-
-	CORBA_exception_init (&ev);
-
-	pc = gtk_object_get_data (GTK_OBJECT (app), "property-control");
-	Bonobo_PropertyControl_notifyAction (pc, 0, Bonobo_PropertyControl_APPLY, &ev);
-	gtk_widget_destroy (app);
-
-	bonobo_object_release_unref (pc, &ev);
-
-	CORBA_exception_free (&ev);
-}
-
-static void
-capplet_cancel_cb (GtkWidget *widget, GtkWidget *app) 
-{
-	CORBA_Environment ev;
-	Bonobo_PropertyControl pc;
-
-	CORBA_exception_init (&ev);
-
-	pc = gtk_object_get_data (GTK_OBJECT (app), "property-control");
-	gtk_widget_destroy (app);
-
-	bonobo_object_release_unref (pc, &ev);
-
-	CORBA_exception_free (&ev);
-}
-
-#ifdef HAVE_BONOBO
-
-/* capplet_control_launch
- *
- * Launch a capplet as a Bonobo control; returns the relevant BonoboWindow or
- * NULL if the capplet could not be launched
- */
-
-GtkWidget *
-capplet_control_launch (const gchar *capplet_name, gchar *window_title)
-{
-	gchar                  *oaf_iid;
-	gchar                  *moniker;
-	gchar                  *tmp;
-	gchar                  *tmp1;
-
-	GtkWidget              *app;
-	GtkWidget              *control;
-	GtkWidget              *dialog;
-	CORBA_Environment       ev;
-
-	Bonobo_PropertyControl  property_control;
-	Bonobo_Control          control_ref;
-	Bonobo_PropertyBag      pb;
-
-	BonoboControlFrame     *cf;
-
-	g_return_val_if_fail (capplet_name != NULL, NULL);
-
-	CORBA_exception_init (&ev);
-
-	tmp = g_strdup (capplet_name);
-	if ((tmp1 = strstr (tmp, "-capplet")) != NULL) *tmp1 = '\0';
-	moniker = g_strconcat ("archive:user-archive#archiverdb:", tmp, NULL);
-	while ((tmp1 = strchr (tmp, '-'))) *tmp1 = '_';
-
-	oaf_iid = g_strconcat ("OAFIID:Bonobo_Control_Capplet_", tmp, NULL);
-	g_free (tmp);
-
-	property_control = bonobo_get_object (oaf_iid, "IDL:Bonobo/PropertyControl:1.0", &ev);
-	g_free (oaf_iid);
-
-	if (BONOBO_EX (&ev) || property_control == CORBA_OBJECT_NIL) {
-		dialog = gnome_error_dialog ("Could not load the capplet.");
-		gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
-		g_free (moniker);
-		return NULL;
-	}
-
-	control_ref = Bonobo_PropertyControl_getControl (property_control, 0, &ev);
-
-	if (BONOBO_EX (&ev) || control_ref == CORBA_OBJECT_NIL) {
-		bonobo_object_release_unref (property_control, &ev);
-		g_free (moniker);
-		return NULL;
-	}
-
-	app = gnome_dialog_new (window_title, GNOME_STOCK_BUTTON_OK,
-				GNOME_STOCK_BUTTON_CANCEL, NULL);
-	gtk_object_set_data (GTK_OBJECT (app), "property-control", property_control);
-	control = bonobo_widget_new_control_from_objref (control_ref, CORBA_OBJECT_NIL);
-
-	if (control == NULL) {
-		g_critical ("Could not create widget from control");
-		gtk_widget_destroy (app);
-		bonobo_object_release_unref (property_control, &ev);
-		app = NULL;
-	} else {
-		gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (app)->vbox), control, TRUE, TRUE, 0);
-
-		cf = bonobo_widget_get_control_frame (BONOBO_WIDGET (control));
-		pb = bonobo_control_frame_get_control_property_bag (cf, &ev);
-		bonobo_property_bag_client_set_value_string (pb, "moniker", moniker, &ev);
-
-		if (BONOBO_EX (&ev)) {
-			dialog = gnome_error_dialog ("Could not load your configuration settings.");
-			gnome_dialog_run_and_close (GNOME_DIALOG (dialog));
-			gtk_widget_destroy (app);
-			bonobo_object_release_unref (property_control, &ev);
-			app = NULL;
-		} else {
-			gnome_dialog_button_connect (GNOME_DIALOG (app), 0, GTK_SIGNAL_FUNC (capplet_ok_cb), app);
-			gnome_dialog_button_connect (GNOME_DIALOG (app), 1, GTK_SIGNAL_FUNC (capplet_cancel_cb), app);
-			gtk_widget_show_all (app);
-		}
-	}
-
-	CORBA_exception_free (&ev);
-	g_free (moniker);
-
-	return app;
-}
-
-#endif /* HAVE_BONOBO */
